@@ -287,7 +287,7 @@ refresh_preview_surfaces (void)
   cairo_surface_t *preview_surface = NULL;
 
   context = gtk_widget_get_style_context (widget);
-  gtk_style_context_get_background_color (context, GTK_STATE_FLAG_NORMAL, &bg);
+  gtk_style_context_get_background_color (context, gtk_style_context_get_state (context), &bg);
 
   /* Like the refresh_pixmaps() function, we may be called before
    * the window is ready. */
@@ -1457,25 +1457,12 @@ game_props_callback (GSimpleAction *action,
   gtk_dialog_run (GTK_DIALOG (pref_dialog));
 }
 
-static void cleanup ()
-{
-  int i = 0;
-
-  for (i = 0; i < G_N_ELEMENTS (preview_images); i++)
-    if (preview_pixbufs[i])
-      g_object_unref (preview_pixbufs[i]);
-
-  g_clear_object (&ball_preimage);
-  g_object_unref (highscores);
-
-}
-
 void
 game_quit_callback (GSimpleAction *action,
                     GVariant *parameter,
                     gpointer user_data)
 {
-  cleanup ();
+  g_application_quit (G_APPLICATION (user_data));
 }
 
 void
@@ -1547,13 +1534,10 @@ init_config (void)
 }
 
 static gboolean
-window_configure_event_cb (GtkWidget *widget, GdkEventConfigure *event)
+window_size_allocate_cb (GtkWidget *widget, GdkRectangle *allocation)
 {
   if (!window_is_maximized && !window_is_fullscreen)
-  {
-    window_width = event->width;
-    window_height = event->height;
-  }
+    gtk_window_get_size (GTK_WINDOW (widget), &window_width, &window_height);
 
   return FALSE;
 }
@@ -1629,7 +1613,7 @@ startup_cb (GApplication *application)
 
   app = GTK_WIDGET (gtk_builder_get_object (builder, "glines_window"));
   gtk_window_set_icon_name (GTK_WINDOW (app), "five-or-more");
-  g_signal_connect (GTK_WINDOW (app), "configure-event", G_CALLBACK (window_configure_event_cb), NULL);
+  g_signal_connect (GTK_WINDOW (app), "size-allocate", G_CALLBACK (window_size_allocate_cb), NULL);
   g_signal_connect (GTK_WINDOW (app), "window-state-event", G_CALLBACK (window_state_event_cb), NULL);
   gtk_window_set_default_size (GTK_WINDOW (app), g_settings_get_int (settings, "window-width"), g_settings_get_int (settings, "window-height"));
   if (g_settings_get_boolean (settings, "window-is-maximized"))
@@ -1692,10 +1676,18 @@ activate_cb (GApplication *application)
 static void
 shutdown_cb (GApplication *application)
 {
+  int i = 0;
+
+  for (i = 0; i < G_N_ELEMENTS (preview_images); i++)
+    if (preview_pixbufs[i])
+      g_object_unref (preview_pixbufs[i]);
+
+  g_clear_object (&ball_preimage);
+  g_object_unref (highscores);
+
   g_settings_set_int (settings, "window-width", window_width);
   g_settings_set_int (settings, "window-height", window_height);
   g_settings_set_boolean (settings, "window-is-maximized", window_is_maximized);
-  cleanup ();
 }
 
 int
